%%%%%%%%%%%%(c) Jan Kabatek, Universiteit van Tilburg %%%%%%%%%%%%%%%%%%%%%
% This is an auxiliary m-file for 7th tutorial of Econometrics 1 for      %
% research masters.                                                       %
%                                                                         %
% Contains: clustered errors, FE, BG & RE estimators                      %
%                                                                         %
% IMPORTANT: before asking me about a specific command in this code(or any%
% code) consult a HELP FILE by parsing:	 help <that command into the      %     
% Matlab console.                                                         %	
%																		  %	
% Version 17.11.2011, j.kabatek@uvt.nl, K606 (should you find any typos,  %
% let me know)                                                            %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear all
tic                                  % gentlemen, start your engines...

%Data Manipulation
[data,names] = xlsread('data.csv');

Y       = data(:,1);
X       = [data(:,2:end)];
X       = [X, ones(length(Y),1)];    % including a constant...

N       = length(Y);
T       = 8;
M       = N/T;


% -----------------------------------------

res             = my_ols(Y,X) ;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    
% Clusters (uses kronecker product, & sparse diagonal matrices) %%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
emat            = repmat(res.resid,1,N);
aux             = (kron(speye(M),ones(T,T)).*emat);
omega           = aux.*aux';

% sparse matrices are more efficient, as they disregard zero-valued cells 
% in the usual matrix operations (products, sums, etc.).

var_b_cl        = (X'*X)\X'*omega*X/(X'*X) ;
se_b_cl         = sqrt(diag(var_b_cl)) ;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    
% FE estimator  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 

X_FE            = [ data(:,2:5) , data(:,7:8) ];
% regressors excluding time-invariant X's

%inefficient way of FE is to include all the individual dummies manually:
    % dum           = kron(eye(M), ones(T,1)) ;
    % X_FE_dum      = [ X_FE dum ] ;
    % res2          = my_ols(Y,X_FE_dum) ;
    % res.beta_hat  = X_FE_dum'*X_FE_dum\(X_FE_dum'*Y) ;
    % res.yhat      = X_FE_dum*res.beta_hat ;

%__________________________________________________________________________
% By de-meaning, we circumvent the hassle with individual dummies:
P_D             = kron(speye(M),ones(T,T).*1/T) ;   % =projection matrix 
% from the book, notice scalar multiplication by 1/T (that's what gets you
% the means, instead of within-group sums)
M_D             = speye(N) - P_D  ;         % =orthogonal projection matrix

rFE.beta_hat    = X_FE'*M_D*X_FE\(X_FE'*M_D*Y) ;
rFE.yhat        = M_D*X_FE*rFE.beta_hat ;
rFE.resid       = M_D*Y-rFE.yhat ; 

rFE.K           = size(X_FE,2);
rFE.N           = length(Y);
rFE.rss         = rFE.resid'*rFE.resid ;
rFE.sig2_hat    = rFE.rss/(rFE.N-rFE.K-M);  %you need to subtract M
% as we are de-meaning the original variables->losing M degrees of freeedom
rFE.sig_hat     = sqrt(rFE.sig2_hat) ; 

sigma2_e        = rFE.sig2_hat;             %this is estimator of sigma^2_e
sigma_e         = rFE.sig_hat

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    
% BG estimator  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

rBG.beta_hat    = X'*P_D*X\(X'*P_D*Y) ;
rBG.yhat        = P_D*X*rBG.beta_hat ;
rBG.resid       = P_D*Y-rBG.yhat ; 

rBG.K           = size(X,2);
rBG.N           = length(Y);
rBG.rss         = rBG.resid'*rBG.resid ;
rBG.sig2_hat    = rBG.rss/((M-rBG.K)*T);  
rBG.sig_hat     = sqrt(rBG.sig2_hat) ; 
sigma2_BG       = rBG.sig2_hat ;

sigma2_v        = sigma2_BG - sigma2_e/T ;  %this is estimator of sigma^2_v
sigma_v         = sqrt(sigma2_v)

% Same result would be achieved by running regression with m observations,
% as discussed in the book:
    % aux = kron(eye(M),[1 , zeros(1,T-1)]);
    % X_BG            = aux*P_D*X ;
    % Y_BG            = aux*P_D*Y ;
    % 
    % rBG.beta_hat    = X_BG'*X_BG\(X_BG'*Y_BG) ;
    % rBG.yhat        = X_BG*rBG.beta_hat ;
    % rBG.resid       = Y_BG-rBG.yhat ; 
    % rBG.K           = size(X,2);
    % rBG.N           = length(Y);
    % rBG.rss         = rBG.resid'*rBG.resid ;
    % rBG.sig2_hat    = rBG.rss/(M-rBG.K);
    %  
    % sigma2_BG2      = rBG.sig2_hat
    % sigma2_v        = sigma2_BG2 - sigma2_e/T 
    % sigma_v         = sqrt(sigma2_v)
    
    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    
% RE estimator  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
lambda          = 1 - ( T*sigma2_v / sigma2_e +1 )^(-1/2) ; 
psi             = speye(N) - lambda * P_D  ; 

% note that psi'*psi is an inverse of omega_hat (-> GLS estimation)

rRE.beta_hat    = X'*psi'*psi*X\(X'*psi'*psi*Y) ;
rRE.yhat        = psi*X*rRE.beta_hat ;
rRE.resid       = psi*Y-rRE.yhat ;

rRE.K           = size(X,2);
rRE.N           = length(Y);
rRE.rss         = rRE.resid'*rRE.resid ;
rRE.sig2_hat    = rRE.rss/(rRE.N-rRE.K);   
% rRE.sig2_hat    = rRE.rss/(rRE.N-rRE.K-M+1);  %this one might be more
% valid, Stata uses the one above, though.
rRE.sig_hat     = sqrt(rRE.sig2_hat) ; 
rRE.var_beta    =(X'*psi'*psi*X)\eye(res.K)*rRE.sig2_hat ;

rRE.se_beta		= sqrt(diag(rRE.var_beta)) ;
rRE.tstat       = rRE.beta_hat./rRE.se_beta ;

rho             = sigma2_v/(sigma2_e+sigma2_v) 

toc                                         % Finish!!!

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
% If you can program these (substantially) faster, let me know! Thanks. %%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
